package com.hw.wms.controller;

import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.hw.common.log.annotation.Log;
import com.hw.common.log.enums.BusinessType;
import com.hw.common.security.annotation.RequiresPermissions;
import com.hw.wms.domain.WmsRawReturn;
import com.hw.wms.service.IWmsRawReturnService;
import com.hw.common.core.web.controller.BaseController;
import com.hw.common.core.web.domain.AjaxResult;
import com.hw.common.core.utils.poi.ExcelUtil;
import com.hw.common.core.web.page.TableDataInfo;

/**
 * 原材料退库记录Controller
 * 
 * @author xs
 * @date 2023-12-20
 */
@RestController
@RequestMapping("/rawreturn")
public class WmsRawReturnController extends BaseController
{
    @Autowired
    private IWmsRawReturnService wmsRawReturnService;

    /**
     * 查询原材料退库记录列表
     */
    @RequiresPermissions("wms:rawreturn:list")
    @GetMapping("/list")
    public TableDataInfo list(WmsRawReturn wmsRawReturn)
    {
        startPage();
        List<WmsRawReturn> list = wmsRawReturnService.selectWmsRawReturnList(wmsRawReturn);
        return getDataTable(list);
    }

    /**
     * 导出原材料退库记录列表
     */
    @RequiresPermissions("wms:rawreturn:export")
    @Log(title = "原材料退库记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, WmsRawReturn wmsRawReturn)
    {
        List<WmsRawReturn> list = wmsRawReturnService.selectWmsRawReturnList(wmsRawReturn);
        ExcelUtil<WmsRawReturn> util = new ExcelUtil<WmsRawReturn>(WmsRawReturn.class);
        util.exportExcel(response, list, "原材料退库记录数据");
    }

    /**
     * 获取原材料退库记录详细信息
     */
    @RequiresPermissions("wms:rawreturn:query")
    @GetMapping(value = "/{rawReturnId}")
    public AjaxResult getInfo(@PathVariable("rawReturnId") Long rawReturnId)
    {
        return success(wmsRawReturnService.selectWmsRawReturnByRawReturnId(rawReturnId));
    }

    /**
     * 新增原材料退库记录
     */
    @RequiresPermissions("wms:rawreturn:add")
    @Log(title = "原材料退库记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody WmsRawReturn wmsRawReturn)
    {
        return toAjax(wmsRawReturnService.insertWmsRawReturn(wmsRawReturn));
    }

    /**
     * 修改原材料退库记录
     */
    @RequiresPermissions("wms:rawreturn:edit")
    @Log(title = "原材料退库记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody WmsRawReturn wmsRawReturn)
    {
        return toAjax(wmsRawReturnService.updateWmsRawReturn(wmsRawReturn));
    }

    /**
     * 删除原材料退库记录
     */
    @RequiresPermissions("wms:rawreturn:remove")
    @Log(title = "原材料退库记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{rawReturnIds}")
    public AjaxResult remove(@PathVariable Long[] rawReturnIds)
    {
        return toAjax(wmsRawReturnService.deleteWmsRawReturnByRawReturnIds(rawReturnIds));
    }
}
